/*
 * Copyright (C) 1999-2022. Christian Heller.
 *
 * This file is part of the Cybernetics Oriented Interpreter (CYBOI).
 *
 * CYBOI is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation, either version 3 of the License,
 * or (at your option) any later version.
 *
 * CYBOI is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CYBOI. If not, see <http://www.gnu.org/licenses/>.
 *
 * Cybernetics Oriented Programming (CYBOP) <http://www.cybop.org/>
 * CYBOP Developers <cybop-developers@nongnu.org>
 *
 * @version CYBOP 0.24.0 2022-12-24
 * @author Christian Heller <christian.heller@cybop.org>
 */

#ifndef MUTEX_ALLOCATOR_SOURCE
#define MUTEX_ALLOCATOR_SOURCE

#include <threads.h> // mtx_t, mtx_init, thrd_error

#include "../../../constant/model/cyboi/log/level_log_cyboi_model.c"
#include "../../../constant/model/cyboi/state/pointer_state_cyboi_model.c"
#include "../../../constant/model/cyboi/state/state_cyboi_model.c"
#include "../../../constant/type/cyboi/state_cyboi_type.c"
#include "../../../executor/memoriser/allocator/array_allocator.c"
#include "../../../logger/logger.c"
#include "../../../variable/symbolic_name/mutex_thread_symbolic_name.c"

/**
 * Allocates the mutex.
 *
 * @param p0 the mutex (pointer reference)
 */
void allocate_mutex(void* p0) {

    if (p0 != *NULL_POINTER_STATE_CYBOI_MODEL) {

        void** m = (void**) p0;

        log_message_terminated((void*) DEBUG_LEVEL_LOG_CYBOI_MODEL, (void*) L"Allocate mutex.");
        //?? fwprintf(stdout, L"Debug: Allocate mutex. p0: %i\n", p0);
        //?? fwprintf(stdout, L"Debug: Allocate mutex. *p0: %i\n", *((int*) p0));

        //
        // Allocate mutex.
        //
        // CAUTION! Due to memory allocation handling, the size MUST NOT
        // be negative or zero, but have at least a value of ONE.
        //
        allocate_array(p0, (void*) PRIMITIVE_STATE_CYBOI_MODEL_COUNT, (void*) MUTEX_THREAD_STATE_CYBOI_TYPE);

        // Cast mutex to correct type.
        mtx_t* t = (mtx_t*) *m;

        // Initialise mutex.
        int r = mtx_init(t, *PLAIN_MUTEX_TYPE_THREAD_SYMBOLIC_NAME);

        // Evaluate mutex object creation result.
        if (r == thrd_error) {

            log_message_terminated((void*) ERROR_LEVEL_LOG_CYBOI_MODEL, (void*) L"Could not allocate mutex. The mutex object creation failed.");
            fwprintf(stdout, L"Error: Could not allocate mutex. The mutex object creation failed. mr: %i\n", r);
        }

    } else {

        log_message_terminated((void*) ERROR_LEVEL_LOG_CYBOI_MODEL, (void*) L"Could not allocate mutex. The mutex is null.");
        fwprintf(stdout, L"Error: Could not allocate mutex. The mutex is null. p0: %i\n", p0);
    }
}

/* MUTEX_ALLOCATOR_SOURCE */
#endif
